/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "main_window.hpp"
#include "version_window.hpp"
#include "memcard/memorycard.hpp"
#include "make_nana_animation.hpp"
#include "memcard/make_memory_card.hpp"
#include "widget/psx_mc_grid.hpp"
#include <nana/gui/filebox.hpp>
#include <iostream>
#include <cassert>
#include <filesystem>

namespace duck {
namespace {
enum class AccelMod { Ctrl, Alt, Shift };

template <AccelMod... Mods> struct SetupStartingAccelKey;
template <> struct SetupStartingAccelKey<> {
	static constexpr nana::accel_key make() { return nana::accel_key(); }
};
template <AccelMod Mod, AccelMod... Mods> struct SetupStartingAccelKey<Mod, Mods...> {
	static constexpr nana::accel_key make() {
		auto retval = SetupStartingAccelKey<Mods...>::make();
		if constexpr (Mod == AccelMod::Ctrl) retval.ctrl = true;
		else if constexpr (Mod == AccelMod::Alt) retval.alt = true;
		else if constexpr (Mod == AccelMod::Shift) retval.shift = true;
		return retval;
	}
};

template <AccelMod... Mods>
constexpr nana::accel_key make_accel_key (char key) {
	nana::accel_key retval = SetupStartingAccelKey<Mods...>::make();
	retval.key = key;
	return retval;
}

template <typename F>
void append_to_menu (nana::form& owner, nana::menu& menu, const std::string& text, const F& func, nana::accel_key accel) {

	menu.append(text, func);
	owner.keyboard_accelerator(accel, [&menu,idx=menu.size()-1](){menu.invoke(idx);});
}
} //unnamed namespace

MainWindow::MainWindow (int icon_size, int icon_fps, const std::string& caption) :
	nana::form(nana::API::make_center(800, 600)),
	m_menubar(*this),
	m_icon_size(icon_size),
	m_icon_fps(icon_fps)
{
	this->caption(caption);
	this->init_menu();

	this->div("vert <menu weight=28><margin=3 <fit slots gap=20>>");
	(*this)["menu"] << m_menubar;
	this->collocate();
}

MainWindow::~MainWindow() noexcept = default;

void MainWindow::init_menu() {
	m_menubar.push_back("&File");

	append_to_menu(
		*this,
		m_menubar.at(0),
		"&Open...",
		[this](nana::menu::item_proxy&){this->open_file_from_dialog();},
		make_accel_key<AccelMod::Ctrl>('o')
	);

	append_to_menu (
		*this,
		m_menubar.at(0),
		"E&xit",
		[](nana::menu::item_proxy&){nana::API::exit();},
		make_accel_key<AccelMod::Ctrl>('q')
	);

	m_menubar.push_back("&Help");
	m_menubar.at(1).append("Version info", [this](nana::menu::item_proxy&) {
		VersionWindow win(*this);
		win.show();
		win.modality();
	});
}

void MainWindow::open_file_from_dialog() {
	using std::filesystem::path;

	nana::filebox fb(*this, true);
	fb.allow_multi_select(true);
	fb.add_filter(("PSX MemoryCard"), ("*.mcr;*.sav;*.srm"));
	fb.add_filter(("All files"), ("*.*"));
	auto files = fb();

	for (const auto& file : files) {
		m_memcards.push_back(mc::psx::make_memory_card(file));
		m_grids.emplace_back(*this, path(file).filename());
#if !defined(NDEBUG)
		m_grids.back().bgcolor(nana::colors::azure);
#endif

		auto& mc = m_memcards.back();
		auto& grid = m_grids.back();
		for (const auto& block : mc) {
			for (std::size_t n = 0; block.has_magic() and n < block.block_count(); ++n) {
				//grid.emplace_back(duck::make_nana_animation(block, m_icon_size, m_icon_size, m_icon_fps));
			}
		}
		(*this)["slots"] << m_grids.back();
	}
	this->collocate();
}
} //namespace duck
