/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "block_grid.hpp"
#include <QGridLayout>
#include <iostream>

namespace duck::widget {
namespace {
	const constexpr int g_icon_spacing = 3;
} //unnamed namespace

BlockGrid::BlockGrid (QWidget* parent) :
	BlockGrid(parent, 3)
{
}

BlockGrid::BlockGrid (QWidget* parent, unsigned int columns) :
	QWidget(parent),
	m_columns(columns),
	m_block_lay(nullptr)
{
	auto block_lay = std::make_unique<QGridLayout>();
	block_lay->setSpacing(g_icon_spacing);
	m_block_lay = block_lay.get();
	m_block_lay->setColumnStretch(m_columns, 1);
	this->setLayout(block_lay.release());
}

BlockGrid::~BlockGrid() noexcept = default;

QSize BlockGrid::minimumSizeHint() const {
	return sizeHint();
}

QSize BlockGrid::sizeHint() const {
	const auto w = icon_width() * m_columns + g_icon_spacing * (1 + m_columns);
	const unsigned int rows = size() / m_columns;
	const auto h = icon_height() * rows + g_icon_spacing * (rows + 1);

	return {static_cast<int>(w), static_cast<int>(h)};
}

unsigned int BlockGrid::icon_width() const {
	return m_icons.empty() ? AnimatedIcon::EmptyIconWidth : m_icons.front()->width();
}

unsigned int BlockGrid::icon_height() const {
	return m_icons.empty() ? AnimatedIcon::EmptyIconHeight : m_icons.front()->height();
}

void BlockGrid::emplace_back (std::unique_ptr<AnimatedIcon>&& anim) {
	const auto& columns = m_columns;
	const auto count = this->size();

	const int x = count % columns;
	const int y = count / columns;

	{
		const auto raw_ptr = anim.release();
		m_block_lay->addWidget(raw_ptr, y, x);
		m_icons.push_back(raw_ptr);
	}

	m_block_lay->setRowStretch(y, 0);
	m_block_lay->setRowStretch(y + 1, 1);
}

std::size_t BlockGrid::size() const {
	return m_icons.size();
}
} //namespace duck::widget
