/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "memcard/memorycard.hpp"
#include "memcard/block.hpp"
#include "memcard/frame.hpp"
#include <iterator>
#include <cassert>

namespace mc::psx {
namespace {
	const constexpr std::size_t MemoryCardSize = 8192 * 16;

	template <bool Const>
	BasicBlock<Const> make_block_for_index (typename BasicBlock<Const>::data_type* data, std::size_t index) {
		const auto new_index = (0 == index ? Block::TOCBlockIndex : index - 1);
		const auto retptr = data + index * Block::size() * Frame::size();
		if (index > 0) {
			return {retptr, BasicFrame<Const>(data + index * Frame::size(), index), new_index};
		}
		else {
			return {data + index, new_index};
		}
	}
} //unnamed namespace

MemoryCard::MemoryCard() :
	m_data(MemoryCardSize)
{
}

MemoryCard::~MemoryCard() = default;

Block MemoryCard::operator[] (std::size_t index) {
	return block(index);
}

Block MemoryCard::block (std::size_t index) {
	assert(index < GameBlocks);
	return make_block_for_index<false>(m_data.data(), index + 1);
}

Block MemoryCard::toc_block() {
	return make_block_for_index<false>(m_data.data(), 0);
}

ConstBlock MemoryCard::operator[] (std::size_t index) const {
	return block(index);
}

ConstBlock MemoryCard::block (std::size_t index) const {
	assert(index < GameBlocks);
	return make_block_for_index<true>(m_data.data(), index + 1);
}

ConstBlock MemoryCard::toc_block() const {
	return make_block_for_index<true>(m_data.data(), 0);
}

ContentInfo MemoryCard::content_info() const {
	return {toc_block().frame(0)};
}

std::size_t MemoryCard::size() const {
	return std::count_if(
		const_iterator(this, 0),
		const_iterator(this, GameBlocks),
		[](const auto& blk) {
			return (blk.available_blocks() & 0xF) != 0 and (blk.available_blocks() & 0xF) != 0xF;
		}
	);
}

auto MemoryCard::begin() -> iterator {
	return iterator(this, 0);
}

auto MemoryCard::end() -> iterator {
	return iterator(this, size());
}

auto MemoryCard::cbegin() const -> const_iterator {
	return const_iterator(this, 0);
}

auto MemoryCard::begin() const -> const_iterator {
	return const_iterator(this, 0);
}

auto MemoryCard::cend() const -> const_iterator {
	return const_iterator(this, size());
}

auto MemoryCard::end() const -> const_iterator {
	return const_iterator(this, size());
}

std::vector<std::size_t> block_group (const MemoryCard& mc, std::size_t index) {
	std::vector<std::size_t> retval;
	retval.reserve(MemoryCard::GameBlocks);

	for (const auto& block : mc) {
		retval.clear();
		std::size_t cur_idx = block.index();
		bool this_run = false;
		ConstBlock cur_block;
		do {
			cur_block = mc[cur_idx];
			this_run |= static_cast<bool>(cur_idx == index);
			retval.push_back(cur_idx);
			cur_idx = cur_block.link_order();
		} while (cur_idx != Block::LastLink);

		if (this_run)
			return retval;
	}
	return {};
}
} //namespace mc::psx
